import 'i18n_crop_rotate_editor.dart';
import 'i18n_emoji_editor.dart';
import 'i18n_filter_editor.dart';
import 'i18n_painting_editor.dart';
import 'i18n_sticker_editor.dart';
import 'i18n_text_editor.dart';
import 'i18n_various.dart';

export 'i18n_various.dart';
export 'i18n_text_editor.dart';
export 'i18n_painting_editor.dart';
export 'i18n_filter_editor.dart';
export 'i18n_emoji_editor.dart';
export 'i18n_sticker_editor.dart';
export 'i18n_crop_rotate_editor.dart';

/// The `I18n` class provides internationalization settings for the image editor
/// and its components. It includes translations and messages for various parts
/// of the editor.
///
/// Usage:
///
/// ```dart
/// I18n i18n = I18n(
///   various: I18nVarious(
///     // Define various translations...
///   ),
///   paintEditor: I18nPaintingEditor(
///     // Define painting editor translations...
///   ),
///   textEditor: I18nTextEditor(
///     // Define text editor translations...
///   ),
///   cropRotateEditor: I18nCropRotateEditor(
///     // Define crop and rotate editor translations...
///   ),
///   filterEditor: I18nFilterEditor(
///     // Define filter editor translations...
///   ),
///   emojiEditor: I18nEmojiEditor(
///     // Define emoji editor translations...
///   ),
///   cancel: 'Cancel',
///   undo: 'Undo',
///   redo: 'Redo',
///   done: 'Done',
///   doneLoadingMsg: 'Changes are being applied',
/// );
/// ```
///
/// Properties:
///
/// - `various`: Translations and messages for various parts of the editor.
///
/// - `paintEditor`: Translations and messages specific to the painting editor.
///
/// - `textEditor`: Translations and messages specific to the text editor.
///
/// - `cropRotateEditor`: Translations and messages specific to the crop and
///   rotate editor.
///
/// - `filterEditor`: Translations and messages specific to the filter editor.
///
/// - `emojiEditor`: Translations and messages specific to the emoji editor.
///
/// - `cancel`: The text for the "Cancel" button.
///
/// - `undo`: The text for the "Undo" action.
///
/// - `redo`: The text for the "Redo" action.
///
/// - `done`: The text for the "Done" action.
///
/// - `remove`: The text for the "Remove" action.
///
/// - `doneLoadingMsg`: The message displayed when changes are being applied.
///
/// Example Usage:
///
/// ```dart
/// I18n i18n = I18n(
///   various: I18nVarious(
///     // Define various translations...
///   ),
///   paintEditor: I18nPaintingEditor(
///     // Define painting editor translations...
///   ),
///   // Access other translations and messages...
/// );
///
/// String cancelText = i18n.cancel;
/// String undoText = i18n.undo;
/// // Access other translations and messages...
/// ```
class I18n {
  /// Translations and messages specific to the painting editor.
  final I18nPaintingEditor paintEditor;

  /// Translations and messages for various parts of the editor.
  final I18nVarious various;

  /// Translations and messages specific to the text editor.
  final I18nTextEditor textEditor;

  /// Translations and messages specific to the filter editor.
  final I18nFilterEditor filterEditor;

  /// Translations and messages specific to the emoji editor.
  final I18nEmojiEditor emojiEditor;

  /// Translations and messages specific to the sticker editor.
  final I18nStickerEditor stickerEditor;

  /// Translations and messages specific to the crop and rotate editor.
  final I18nCropRotateEditor cropRotateEditor;

  /// Message displayed while changes are being applied.
  final String doneLoadingMsg;

  /// Text for the "Cancel" action.
  final String cancel;

  /// Text for the "Undo" action.
  final String undo;

  /// Text for the "Redo" action.
  final String redo;

  /// Text for the "Done" action.
  final String done;

  /// Text for the "Remove" action.
  final String remove;

  /// Creates an instance of [I18n] with customizable internationalization settings.
  ///
  /// You can provide translations and messages for various components of the
  /// Image Editor by specifying the corresponding [I18n] subclasses. If a
  /// specific translation is not provided, default values will be used.
  ///
  /// The [cancel], [undo], [redo], and [done] parameters allow you to specify
  /// the text for the respective buttons in the Image Editor interface.
  ///
  /// The [doneLoadingMsg] parameter is used to display a message while changes
  /// are being applied.
  ///
  /// Example:
  ///
  /// ```dart
  /// I18n(
  ///   various: I18nVarious(
  ///     // Custom translations and settings for various components
  ///   ),
  ///   paintEditor: I18nPaintingEditor(
  ///     // Custom translations and settings for the Painting Editor
  ///   ),
  ///   textEditor: I18nTextEditor(
  ///     // Custom translations and settings for the Text Editor
  ///   ),
  ///   // Additional settings for other components...
  ///   cancel: 'Cancel',
  ///   undo: 'Undo',
  ///   redo: 'Redo',
  ///   done: 'Done',
  ///   remove: 'Remove',
  ///   doneLoadingMsg: 'Changes are being applied',
  /// )
  /// ```
  const I18n({
    this.paintEditor = const I18nPaintingEditor(),
    this.textEditor = const I18nTextEditor(),
    this.cropRotateEditor = const I18nCropRotateEditor(),
    this.filterEditor = const I18nFilterEditor(),
    this.emojiEditor = const I18nEmojiEditor(),
    this.stickerEditor = const I18nStickerEditor(),
    this.various = const I18nVarious(),
    this.cancel = 'Cancel',
    this.undo = 'Undo',
    this.redo = 'Redo',
    this.done = 'Done',
    this.remove = 'Remove',
    this.doneLoadingMsg = 'Changes are being applied',
  });
}
